1;	#for octave to recognize this as script file and load all functions
#*******************************************************************************
# BEAMSIM - creates the initial beam structure matrix:
#
#    B0.r,                                     spatial coordinates r=[x,y,z] [m]
#    B0.k,                       wave unit vectors components k=[kx,ky,kz] [1/m]
#    B0.e,                         electric unit vectors components e=[ex,ey,ez]
#    B0.n,                        vectors normal to reference plane n=[nx,ny,nz]
#    B0.s,                                        Stokes parameters s=[P1,P2,P3]
#    B0.ene,                                                        energy [keV]
#    B0.wgt,                                                              weight
#    B0.loc                                                             location
#    B0.pr,                                        process type pr=[p1,p2,p3...]
#
#*******************************************************************************
function[B0]=beamsim(nof,nsc)
    global src ver;
                              # total polarization intensity test and adjustment
    P1=src.P1;  P2=src.P2;  P3=src.P3;                       # Stokes parameters
    if sqrt(P1^2+P2^2+P3^2)>1
        disp('Total polarization intensity I > 1');
        disp('Please correct the Stokes parameters !');    abort;
    elseif sqrt(P1^2+P2^2+P3^2)<1
        P3=sqrt(1-P1^2-P2^2);        # adjusting circular polarization intensity
    endif

    txt=['P1=' num2str(P1) ',  P2=' num2str(P2) ',  P3=' num2str(P3)];
    disp(['_______ BEAMSIM:  ' txt]);
    
    switch src.type                                   # select source-slit shape
    case 'rect'
        xx=(rand(nof,1)-0.5)*src.width;
        yx=ones(nof,1)*src.dist;
        zx=(rand(nof,1)-0.5)*src.height;
        rx0=[xx,yx,zx];                     # photon's coordinates in source [m]
    
        xs=(rand(nof,1)-0.5)*src.slwidth;    
        ys=ones(nof,1)*src.sldist;
        zs=(rand(nof,1)-0.5)*src.slheight;
        rs0=[xs,ys,zs];                       # photon's coordinates in slit [m]
    
    case 'circ'
        xx=(rand(2*nof,1)-0.5)*src.width;
        yx=ones(2*nof,1)*src.dist;
        zx=(rand(2*nof,1)-0.5)*src.width;
        rx=[xx,yx,zx];
        insource=sqrt(xx.^2+zx.^2)<src.width/2;            # in source condition
        rx=rx(insource,:);
        rx0=rx(1:nof,:);                                # reduce to a nof number

        xs=(rand(2*nof,1)-0.5)*src.slwidth;    
        ys=ones(2*nof,1)*src.sldist;
        zs=(rand(2*nof,1)-0.5)*src.slwidth;
        rs=[xs,ys,zs];                        # photon's coordinates in slit [m]
        incolim=sqrt(xs.^2+zs.^2)<src.slwidth/2;             # in slit condition
        rs=rs(incolim,:);
        rs0=rs(1:nof,:);                                # reduce to a nof number

    otherwise
        disp('Undefined source-slit shape!');   return;
    endswitch

    rx=rotxyz(rx0,src.rotx,src.roty,src.rotz);
    rs=rotxyz(rs0,src.rotx,src.roty,src.rotz);
    
    k=rs-rx;                                                      # wave vectors
    k(k==0)=1d-12;                                        # correct components=0
    k=vecnorm(k,1);                               # normalization to unit vector
    ene=randn(nof,1)*src.spread+src.energy;                     # energies [keV]
    wgt=ones(nof,1);                                           # initial weights
    loc=zeros(nof,1);                            # initial location - vacuum (0)
    pr=zeros(nof,nsc);                                     # 1st - nsc processes
    
    switch ver                                           # beam structure matrix
    case '1.0'
        B0=struct('r',rx,'k',k,'ene',ene,'wgt',wgt,'loc',loc,'pr',pr);
    
    case '2.0'                            # calculation of electric unit vectors
        ea=sqrt((1-P1)/2);      # polarization elipse parameter a - along Z axis
        eb=sqrt((1+P1)/2);      # polarization elipse parameter b - along X axis
        alfa=rand(nof,1)*2*pi;           # random angle for polarization ellipse
        ex=ea*cos(alfa);                            # electric vector components
        ey=zeros(nof,1);
        ez=eb*sin(alfa);
        e=[ex,ey,ez];
                       # making e vector _|_ to k0 by adjusting its y component:
        k0=rs0-rx0;
        e(:,2)=-(k0(:,1).*e(:,1)+k0(:,3).*e(:,3))./k0(:,2);
        e=rx0+e;
        e=rotxyz(e,src.rotx,src.roty,src.rotz);
        e=e-rx;
        e=vecnorm(e,1);
        B0=struct('r',rx,'k',k,'ene',ene,'wgt',wgt,'loc',loc,'pr',pr,'e',e);

    case '3.0'             # calculation of normal vectors and Stokes parameters
        n(1:nof,1)=0;  n(1:nof,2)=0;  n(1:nof,3)=1;      # starting from (0,0,1)
        rax(:,1)=ones(nof,1); rax(:,2:3)=zeros(nof,2);   # 1st rotation axis (X)
        k0=rs0-rx0;
        rang=asin(k0(:,3));                                # 1st rotation angles
        n=vecrot(n,rax,rang);
        rax(:,1:2)=zeros(nof,2);  rax(:,3)=ones(nof,1);  # 2nd rotation axis (Z)
        rang=atan(k0(:,1)./k0(:,2));                       # 2nd rotation angles
        n=vecrot(n,rax,rang);
        nx=vecnorm(n,1);
        n=rx0+n;                                      # normal vector's rotation
        n=rotxyz(n,src.rotx,src.roty,src.rotz);
        n=n-rx;
        s(1:nof,1)=P1;  s(1:nof,2)=P2;  s(1:nof,3)=P3;       # Stokes parameters
      B0=struct('r',rx,'k',k,'ene',ene,'wgt',wgt,'loc',loc,'pr',pr,'n',n,'s',s);
    
    otherwise
        disp('BEAMSIM: Wrong MUSCAT version!');    return;
    endswitch
# test 1 (for ver. 2.0):
#figure;  hold on;  plot_vec('k',B0,1);  plot_vec('e',B0,1);  axis("equal");
# test 2 (for ver. 3.0):
#figure;  hold on;  plot_vec('k',B0,1);  plot_vec('n',B0,1);  axis("equal");
endfunction
#*******************************************************************************
