1;	#for octave to recognize this as script file and load all functions
#*******************************************************************************
# 1) RANDIR - random change of the wave unit vectors directions
#             into the whole solid angle
#        theta - scattering angles
#        omega - angles between electric vectors
#        fi - angles between reference and scattering planes
#*******************************************************************************
function [B1,theta]=randir1(B0)                                # for MUSCAT v1.0
    B1=B0;
    nof=size(B1.r,1);
#    xtheta=acos(rand(nof,1)*2-1);                      # old method from SCILAB
#    xphi=rand(nof,1)*2*pi;
#    k1=[sin(xtheta).*cos(xphi),sin(xtheta).*sin(xphi),cos(xtheta)];
    k1=randn(nof,3);                      # new faster method by Michael Leitner              
    B1.k=vecnorm(k1,1);                                 # new wave unit vectors
    theta=acos(dot(B0.k',B1.k')');                      # scattering angle [rad]
endfunction
#*******************************************************************************
function [B1,theta,omega]=randir2(B0)                          # for MUSCAT v2.0
    [B1,theta]=randir1(B0);
    [el,omega]=poldir(B0.k,B0.e,B1.k);
    B1.e=el;                                         # new electric unit vectors
endfunction
#*******************************************************************************
function [B1,theta,fi]=randir3(B0)                             # for MUSCAT v3.0
    [B1,theta]=randir1(B0);
    k0=B0.k;  k1=B1.k;
    n0=B0.n;  n1=cross(k1,k0);
    fi=acos(dot(n1',n0')');      # angles between reference and scattering plane
    # scattering plane become a new reference plane, so change the normal to
    # reference plane:
    B1.n=n1;
endfunction
#*******************************************************************************

#*******************************************************************************
# 2) DETDIR - change unit wave vectors directions into the detector direction
#        theta - scattering angles
#        omega - angles between electric vectors
#        fi - angles between reference and scattering planes
#*******************************************************************************
function [B1,theta]=detdir1(B0)                                # for MUSCAT v1.0
    global dtctr;
    B1=B0;
    nof=size(B1.r,1);
    B1=rotdet(B1);                        # beam rot. 2 det. dir. (along Z axis)
    
    switch dtctr.type                                            # detector type
    case 'circ'                                          # for circular detector
        r1x=(rand(2*nof,1)-0.5)*dtctr.colx;
        r1y=(rand(2*nof,1)-0.5)*dtctr.colx;
        r1z=ones(2*nof,1)*dtctr.inpos;
        r1=[r1x,r1y,r1z];                       # coefficients in detector plane
        indet=sqrt(r1x.^2+r1y.^2)<(dtctr.colx/2);        # in detector condition
        r1=r1(indet,:);
        r1=r1(1:nof,:);                                 # reduce to a nof number
        k1=r1-B1.r;                             # new wave vector (unnormalized)
        ld=sqrt(sum(k1.^2,2));                            # dist. 2 detector [m]
        w=((dtctr.colx/2)^2)./(4*ld.^2);                   # weights calculation
    case 'rect'                                       # for rectangular detector
        r1x=(rand(nof,1)-0.5)*dtctr.colx;
        r1y=(rand(nof,1)-0.5)*dtctr.coly;
        r1z=ones(nof,1)*dtctr.inpos;
        r1=[r1x,r1y,r1z];                   # coefficients in detector rectangle
        k1=r1-B1.r;                             # new wave vector (unnormalized)
        ld=sqrt(sum(k1.^2,2));                            # dist. 2 detector [m]
        w=(dtctr.colx*dtctr.coly)./(4*pi*ld.^2);           # weights calculation
    endswitch
    
    B1.k=vecnorm(k1,1);                           # insert new wave unit vectors
    B1.wgt=B1.wgt.*w;                                     # weights modification
    B1=rotback(B1);                                              # back rotation
    theta=acos(dot(B0.k',B1.k')');                      # scattering angle [rad]
endfunction
#*******************************************************************************
function [B1,theta,omega]=detdir2(B0)                          # for MUSCAT v2.0
    [B1,theta]=detdir1(B0);
    [el,omega]=poldir(B0.k,B0.e,B1.k);
    B1.e=el;
endfunction
#*******************************************************************************
function [B1,theta,fi]=detdir3(B0)                             # for MUSCAT v3.0
    [B1,theta]=detdir1(B0);
    k0=B0.k;  k1=B1.k;
    n0=B0.n;  n1=cross(k1,k0);
    fi=acos(dot(n1',n0')');      # angles between reference and scattering plane
    # scattering plane become a new reference plane, so change the normal to
    # reference plane:
    B1.n=n1;
endfunction
#*******************************************************************************

#*******************************************************************************
# 3) POLDIR - change polarization direction e0 (electric vector) according
#             to new wave vectors (k1) directions, scattering angles (theta)
#             and angles between electric vectors (omega) are calculated
#*******************************************************************************
function [p1,omega]=poldir(k0,e0,k1)                           # for MUSCAT v2.0
# unit vectors normal to incident plane (plane 0)
    n0=cross(k0,e0);  n0=vecnorm(n0,1);
# unit vectors normal to scattering plane (plane 1)
    n1=cross(k1,e0);    n1=vecnorm(n1,1);
# angles between planes 0 & 1 (n0 & n1):
# if vector product direction is the same as electric vector -> angle between
# planes is from 0 to pi, if oposite -> angle is from pi to 2*pi, this is
# tested by the sign of the dot product
    vp=cross(n0,n1);                                 # vector product of n0 & n1
    dp=dot(e0',vp')';
    kos=dot(n0',n1')';
    alfa=real(acos(kos));
    alfa(dp<0)=2*pi-alfa(dp<0);                       # modification for alfa>pi
    k1x=vecrot(k0,e0,alfa);                              # k0 rotation around e0
# angles between k1x & k1 wave unit vectors:
# if vector product direction is the same as n1 -> angle between k1 and final
# wave vector is from 0 to pi, if oposite -> angle is from pi to 2*pi,
# this is tested by the sign of the dot product
    vp=cross(k1x,k1);                 # vector product of k1 & final wave vector
    dp=dot(n1',vp')';
    kos=dot(k1x',k1')';
    alfa=real(acos(kos));
    alfa(dp<0)=2*pi-alfa(dp<0);                       # modification for alfa>pi
    k1=vecrot(k1x,n1,alfa);                              # k1 rotation around n1
# seems to work fine so we can rotate the electric vector finally:
    p1=vecrot(e0,n1,alfa);
    kos=dot(e0',p1')';
    omega=real(acos(kos));                               # angle between e0 & p1
#    disp(max(dot(p1',k1')'));                              # perpendicular test
endfunction
#*******************************************************************************
